@@ -13,11 +13,15 @@ from ..constants import (
     SUPPORTED_PYTHON_VERSIONS,
 )
 
+from ..util import (
+    REMOTE_ARCHITECTURES,
+)
+
 from ..completion import (
-    DOCKER_COMPLETION,
-    NETWORK_COMPLETION,
-    REMOTE_COMPLETION,
-    WINDOWS_COMPLETION,
+    docker_completion,
+    network_completion,
+    remote_completion,
+    windows_completion,
     filter_completion,
 )
 
@@ -53,12 +57,17 @@ from ..config import (
 
 from .completers import (
     complete_choices,
+    register_completer,
 )
 
 from .converters import (
     key_value_type,
 )
 
+from .epilog import (
+    get_epilog,
+)
+
 from ..ci import (
     get_ci_provider,
 )
@@ -72,11 +81,11 @@ class ControllerMode(enum.Enum):
 
 
 def add_environments(
-        parser,  # type: argparse.ArgumentParser
-        completer,  # type: CompositeActionCompletionFinder
-        controller_mode,  # type: ControllerMode
-        target_mode,  # type: TargetMode
-):  # type: (...) -> None
+        parser: argparse.ArgumentParser,
+        completer: CompositeActionCompletionFinder,
+        controller_mode: ControllerMode,
+        target_mode: TargetMode,
+) -> None:
     """Add arguments for the environments used to run ansible-test and commands it invokes."""
     no_environment = controller_mode == ControllerMode.NO_DELEGATION and target_mode == TargetMode.NO_TARGETS
 
@@ -98,13 +107,15 @@ def add_environments(
     if not get_ci_provider().supports_core_ci_auth():
         sections.append('Remote provisioning options have been hidden since no Ansible Core CI API key was found.')
 
+    sections.append(get_epilog(completer))
+
     parser.formatter_class = argparse.RawDescriptionHelpFormatter
     parser.epilog = '\n\n'.join(sections)
 
 
 def add_global_options(
-        parser,  # type: argparse.ArgumentParser
-        controller_mode,  # type: ControllerMode
+        parser: argparse.ArgumentParser,
+        controller_mode: ControllerMode,
 ):
     """Add global options for controlling the test environment that work with both the legacy and composite options."""
     global_parser = t.cast(argparse.ArgumentParser, parser.add_argument_group(title='global environment arguments'))
@@ -145,11 +156,11 @@ def add_global_options(
 
 
 def add_composite_environment_options(
-        parser,  # type: argparse.ArgumentParser
-        completer,  # type: CompositeActionCompletionFinder
-        controller_mode,  # type: ControllerMode
-        target_mode,  # type: TargetMode
-):  # type: (...) -> t.List[t.Type[CompositeAction]]
+        parser: argparse.ArgumentParser,
+        completer: CompositeActionCompletionFinder,
+        controller_mode: ControllerMode,
+        target_mode: TargetMode,
+) -> list[t.Type[CompositeAction]]:
     """Add composite options for controlling the test environment."""
     composite_parser = t.cast(argparse.ArgumentParser, parser.add_argument_group(
         title='composite environment arguments (mutually exclusive with "environment arguments" above)'))
@@ -159,9 +170,9 @@ def add_composite_environment_options(
         help=argparse.SUPPRESS,
     )
 
-    action_types = []  # type: t.List[t.Type[CompositeAction]]
+    action_types: list[t.Type[CompositeAction]] = []
 
-    def register_action_type(action_type):  # type: (t.Type[CompositeAction]) -> t.Type[CompositeAction]
+    def register_action_type(action_type: t.Type[CompositeAction]) -> t.Type[CompositeAction]:
         """Register the provided composite action type and return it."""
         action_types.append(action_type)
         return action_type
@@ -169,40 +180,40 @@ def add_composite_environment_options(
     if controller_mode == ControllerMode.NO_DELEGATION:
         composite_parser.set_defaults(controller=None)
     else:
-        composite_parser.add_argument(
+        register_completer(composite_parser.add_argument(
             '--controller',
             metavar='OPT',
             action=register_action_type(DelegatedControllerAction if controller_mode == ControllerMode.DELEGATED else OriginControllerAction),
             help='configuration for the controller',
-        ).completer = completer.completer
+        ), completer.completer)
 
     if target_mode == TargetMode.NO_TARGETS:
         composite_parser.set_defaults(targets=[])
     elif target_mode == TargetMode.SHELL:
         group = composite_parser.add_mutually_exclusive_group()
 
-        group.add_argument(
+        register_completer(group.add_argument(
             '--target-posix',
             metavar='OPT',
             action=register_action_type(PosixSshTargetAction),
             help='configuration for the target',
-        ).completer = completer.completer
+        ), completer.completer)
 
         suppress = None if get_ci_provider().supports_core_ci_auth() else argparse.SUPPRESS
 
-        group.add_argument(
+        register_completer(group.add_argument(
             '--target-windows',
             metavar='OPT',
             action=WindowsSshTargetAction if suppress else register_action_type(WindowsSshTargetAction),
             help=suppress or 'configuration for the target',
-        ).completer = completer.completer
+        ), completer.completer)
 
-        group.add_argument(
+        register_completer(group.add_argument(
             '--target-network',
             metavar='OPT',
             action=NetworkSshTargetAction if suppress else register_action_type(NetworkSshTargetAction),
             help=suppress or 'configuration for the target',
-        ).completer = completer.completer
+        ), completer.completer)
     else:
         if target_mode.multiple_pythons:
             target_option = '--target-python'
@@ -224,35 +235,36 @@ def add_composite_environment_options(
 
         target_action = target_actions[target_mode]
 
-        composite_parser.add_argument(
+        register_completer(composite_parser.add_argument(
             target_option,
             metavar='OPT',
             action=register_action_type(target_action),
             help=target_help,
-        ).completer = completer.completer
+        ), completer.completer)
 
     return action_types
 
 
 def add_legacy_environment_options(
-        parser,  # type: argparse.ArgumentParser
-        controller_mode,  # type: ControllerMode
-        target_mode,  # type: TargetMode
+        parser: argparse.ArgumentParser,
+        controller_mode: ControllerMode,
+        target_mode: TargetMode,
 ):
     """Add legacy options for controlling the test environment."""
-    # noinspection PyTypeChecker
-    environment = parser.add_argument_group(
-        title='environment arguments (mutually exclusive with "composite environment arguments" below)')  # type: argparse.ArgumentParser
+    environment: argparse.ArgumentParser = parser.add_argument_group(  # type: ignore[assignment]  # real type private
+        title='environment arguments (mutually exclusive with "composite environment arguments" below)')
 
     add_environments_python(environment, target_mode)
     add_environments_host(environment, controller_mode, target_mode)
 
 
 def add_environments_python(
-        environments_parser,  # type: argparse.ArgumentParser
-        target_mode,  # type: TargetMode
-):  # type: (...) -> None
+        environments_parser: argparse.ArgumentParser,
+        target_mode: TargetMode,
+) -> None:
     """Add environment arguments to control the Python version(s) used."""
+    python_versions: tuple[str, ...]
+
     if target_mode.has_python:
         python_versions = SUPPORTED_PYTHON_VERSIONS
     else:
@@ -273,13 +285,12 @@ def add_environments_python(
 
 
 def add_environments_host(
-        environments_parser,  # type: argparse.ArgumentParser
-        controller_mode,  # type: ControllerMode
-        target_mode  # type: TargetMode
-):  # type: (...) -> None
+        environments_parser: argparse.ArgumentParser,
+        controller_mode: ControllerMode,
+        target_mode: TargetMode,
+) -> None:
     """Add environment arguments for the given host and argument modes."""
-    # noinspection PyTypeChecker
-    environments_exclusive_group = environments_parser.add_mutually_exclusive_group()  # type: argparse.ArgumentParser
+    environments_exclusive_group: argparse.ArgumentParser = environments_parser.add_mutually_exclusive_group()  # type: ignore[assignment]  # real type private
 
     add_environment_local(environments_exclusive_group)
     add_environment_venv(environments_exclusive_group, environments_parser)
@@ -296,31 +307,31 @@ def add_environments_host(
 
 
 def add_environment_network(
-    environments_parser,  # type: argparse.ArgumentParser
-):  # type: (...) -> None
+    environments_parser: argparse.ArgumentParser,
+) -> None:
     """Add environment arguments for running on a windows host."""
-    environments_parser.add_argument(
+    register_completer(environments_parser.add_argument(
         '--platform',
         metavar='PLATFORM',
         action='append',
         help='network platform/version',
-    ).completer = complete_network_platform
+    ), complete_network_platform)
 
-    environments_parser.add_argument(
+    register_completer(environments_parser.add_argument(
         '--platform-collection',
         type=key_value_type,
         metavar='PLATFORM=COLLECTION',
         action='append',
         help='collection used to test platform',
-    ).completer = complete_network_platform_collection
+    ), complete_network_platform_collection)
 
-    environments_parser.add_argument(
+    register_completer(environments_parser.add_argument(
         '--platform-connection',
         type=key_value_type,
         metavar='PLATFORM=CONNECTION',
         action='append',
         help='connection used to test platform',
-    ).completer = complete_network_platform_connection
+    ), complete_network_platform_connection)
 
     environments_parser.add_argument(
         '--inventory',
@@ -330,15 +341,15 @@ def add_environment_network(
 
 
 def add_environment_windows(
-        environments_parser,  # type: argparse.ArgumentParser
-):  # type: (...) -> None
+        environments_parser: argparse.ArgumentParser,
+) -> None:
     """Add environment arguments for running on a windows host."""
-    environments_parser.add_argument(
+    register_completer(environments_parser.add_argument(
         '--windows',
         metavar='VERSION',
         action='append',
         help='windows version',
-    ).completer = complete_windows
+    ), complete_windows)
 
     environments_parser.add_argument(
         '--inventory',
@@ -348,8 +359,8 @@ def add_environment_windows(
 
 
 def add_environment_local(
-        exclusive_parser,  # type: argparse.ArgumentParser
-):  # type: (...) -> None
+        exclusive_parser: argparse.ArgumentParser,
+) -> None:
     """Add environment arguments for running on the local (origin) host."""
     exclusive_parser.add_argument(
         '--local',
@@ -359,9 +370,9 @@ def add_environment_local(
 
 
 def add_environment_venv(
-        exclusive_parser,  # type: argparse.ArgumentParser
-        environments_parser,  # type: argparse.ArgumentParser
-):  # type: (...) -> None
+        exclusive_parser: argparse.ArgumentParser,
+        environments_parser: argparse.ArgumentParser,
+) -> None:
     """Add environment arguments for running in ansible-test managed virtual environments."""
     exclusive_parser.add_argument(
         '--venv',
@@ -376,9 +387,9 @@ def add_environment_venv(
 
 
 def add_global_docker(
-        parser,  # type: argparse.ArgumentParser
-        controller_mode,  # type: ControllerMode
-):  # type: (...) -> None
+        parser: argparse.ArgumentParser,
+        controller_mode: ControllerMode,
+) -> None:
     """Add global options for Docker."""
     if controller_mode != ControllerMode.DELEGATED:
         parser.set_defaults(
@@ -386,6 +397,8 @@ def add_global_docker(
             docker_network=None,
             docker_terminate=None,
             prime_containers=False,
+            dev_systemd_debug=False,
+            dev_probe_cgroups=None,
         )
 
         return
@@ -417,25 +430,43 @@ def add_global_docker(
         help='download containers without running tests',
     )
 
+    # Docker support isn't related to ansible-core-ci.
+    # However, ansible-core-ci support is a reasonable indicator that the user may need the `--dev-*` options.
+    suppress = None if get_ci_provider().supports_core_ci_auth() else argparse.SUPPRESS
+
+    parser.add_argument(
+        '--dev-systemd-debug',
+        action='store_true',
+        help=suppress or 'enable systemd debugging in containers',
+    )
+
+    parser.add_argument(
+        '--dev-probe-cgroups',
+        metavar='DIR',
+        nargs='?',
+        const='',
+        help=suppress or 'probe container cgroups, with optional log dir',
+    )
+
 
 def add_environment_docker(
-        exclusive_parser,  # type: argparse.ArgumentParser
-        environments_parser,  # type: argparse.ArgumentParser
-        target_mode,  # type: TargetMode
-):  # type: (...) -> None
+        exclusive_parser: argparse.ArgumentParser,
+        environments_parser: argparse.ArgumentParser,
+        target_mode: TargetMode,
+) -> None:
     """Add environment arguments for running in docker containers."""
     if target_mode in (TargetMode.POSIX_INTEGRATION, TargetMode.SHELL):
-        docker_images = sorted(filter_completion(DOCKER_COMPLETION))
+        docker_images = sorted(filter_completion(docker_completion()))
     else:
-        docker_images = sorted(filter_completion(DOCKER_COMPLETION, controller_only=True))
+        docker_images = sorted(filter_completion(docker_completion(), controller_only=True))
 
-    exclusive_parser.add_argument(
+    register_completer(exclusive_parser.add_argument(
         '--docker',
         metavar='IMAGE',
         nargs='?',
         const='default',
         help='run from a docker container',
-    ).completer = functools.partial(complete_choices, docker_images)
+    ), functools.partial(complete_choices, docker_images))
 
     environments_parser.add_argument(
         '--docker-privileged',
@@ -459,9 +490,9 @@ def add_environment_docker(
 
 
 def add_global_remote(
-        parser,  # type: argparse.ArgumentParser
-        controller_mode,  # type: ControllerMode
-):  # type: (...) -> None
+        parser: argparse.ArgumentParser,
+        controller_mode: ControllerMode,
+) -> None:
     """Add global options for remote instances."""
     if controller_mode != ControllerMode.DELEGATED:
         parser.set_defaults(
@@ -474,12 +505,12 @@ def add_global_remote(
 
     suppress = None if get_ci_provider().supports_core_ci_auth() else argparse.SUPPRESS
 
-    parser.add_argument(
+    register_completer(parser.add_argument(
         '--remote-stage',
         metavar='STAGE',
         default='prod',
         help=suppress or 'remote stage to use: prod, dev',
-    ).completer = complete_remote_stage
+    ), complete_remote_stage)
 
     parser.add_argument(
         '--remote-endpoint',
@@ -498,10 +529,10 @@ def add_global_remote(
 
 
 def add_environment_remote(
-        exclusive_parser,  # type: argparse.ArgumentParser
-        environments_parser,  # type: argparse.ArgumentParser
-        target_mode,  # type: TargetMode
-):  # type: (...) -> None
+        exclusive_parser: argparse.ArgumentParser,
+        environments_parser: argparse.ArgumentParser,
+        target_mode: TargetMode,
+) -> None:
     """Add environment arguments for running in ansible-core-ci provisioned remote virtual machines."""
     if target_mode == TargetMode.POSIX_INTEGRATION:
         remote_platforms = get_remote_platform_choices()
@@ -512,11 +543,11 @@ def add_environment_remote(
 
     suppress = None if get_ci_provider().supports_core_ci_auth() else argparse.SUPPRESS
 
-    exclusive_parser.add_argument(
+    register_completer(exclusive_parser.add_argument(
         '--remote',
         metavar='NAME',
         help=suppress or 'run from a remote instance',
-    ).completer = functools.partial(complete_choices, remote_platforms)
+    ), functools.partial(complete_choices, remote_platforms))
 
     environments_parser.add_argument(
         '--remote-provider',
@@ -525,50 +556,57 @@ def add_environment_remote(
         help=suppress or 'remote provider to use: %(choices)s',
     )
 
+    environments_parser.add_argument(
+        '--remote-arch',
+        metavar='ARCH',
+        choices=REMOTE_ARCHITECTURES,
+        help=suppress or 'remote arch to use: %(choices)s',
+    )
 
-def complete_remote_stage(prefix, **_):  # type: (str, ...) -> t.List[str]
+
+def complete_remote_stage(prefix: str, **_) -> list[str]:
     """Return a list of supported stages matching the given prefix."""
     return [stage for stage in ('prod', 'dev') if stage.startswith(prefix)]
 
 
-def complete_windows(prefix, parsed_args, **_):  # type: (str, argparse.Namespace, ...) -> t.List[str]
+def complete_windows(prefix: str, parsed_args: argparse.Namespace, **_) -> list[str]:
     """Return a list of supported Windows versions matching the given prefix, excluding versions already parsed from the command line."""
     return [i for i in get_windows_version_choices() if i.startswith(prefix) and (not parsed_args.windows or i not in parsed_args.windows)]
 
 
-def complete_network_platform(prefix, parsed_args, **_):  # type: (str, argparse.Namespace, ...) -> t.List[str]
+def complete_network_platform(prefix: str, parsed_args: argparse.Namespace, **_) -> list[str]:
     """Return a list of supported network platforms matching the given prefix, excluding platforms already parsed from the command line."""
-    images = sorted(filter_completion(NETWORK_COMPLETION))
+    images = sorted(filter_completion(network_completion()))
 
     return [i for i in images if i.startswith(prefix) and (not parsed_args.platform or i not in parsed_args.platform)]
 
 
-def complete_network_platform_collection(prefix, parsed_args, **_):  # type: (str, argparse.Namespace, ...) -> t.List[str]
+def complete_network_platform_collection(prefix: str, parsed_args: argparse.Namespace, **_) -> list[str]:
     """Return a list of supported network platforms matching the given prefix, excluding collection platforms already parsed from the command line."""
     left = prefix.split('=')[0]
-    images = sorted(set(image.platform for image in filter_completion(NETWORK_COMPLETION).values()))
+    images = sorted(set(image.platform for image in filter_completion(network_completion()).values()))
 
     return [i + '=' for i in images if i.startswith(left) and (not parsed_args.platform_collection or i not in [x[0] for x in parsed_args.platform_collection])]
 
 
-def complete_network_platform_connection(prefix, parsed_args, **_):  # type: (str, argparse.Namespace, ...) -> t.List[str]
+def complete_network_platform_connection(prefix: str, parsed_args: argparse.Namespace, **_) -> list[str]:
     """Return a list of supported network platforms matching the given prefix, excluding connection platforms already parsed from the command line."""
     left = prefix.split('=')[0]
-    images = sorted(set(image.platform for image in filter_completion(NETWORK_COMPLETION).values()))
+    images = sorted(set(image.platform for image in filter_completion(network_completion()).values()))
 
     return [i + '=' for i in images if i.startswith(left) and (not parsed_args.platform_connection or i not in [x[0] for x in parsed_args.platform_connection])]
 
 
-def get_remote_platform_choices(controller=False):  # type: (bool) -> t.List[str]
+def get_remote_platform_choices(controller: bool = False) -> list[str]:
     """Return a list of supported remote platforms matching the given prefix."""
-    return sorted(filter_completion(REMOTE_COMPLETION, controller_only=controller))
+    return sorted(filter_completion(remote_completion(), controller_only=controller))
 
 
-def get_windows_platform_choices():  # type: () -> t.List[str]
+def get_windows_platform_choices() -> list[str]:
     """Return a list of supported Windows versions matching the given prefix."""
-    return sorted(f'windows/{windows.version}' for windows in filter_completion(WINDOWS_COMPLETION).values())
+    return sorted(f'windows/{windows.version}' for windows in filter_completion(windows_completion()).values())
 
 
-def get_windows_version_choices():  # type: () -> t.List[str]
+def get_windows_version_choices() -> list[str]:
     """Return a list of supported Windows versions."""
-    return sorted(windows.version for windows in filter_completion(WINDOWS_COMPLETION).values())
+    return sorted(windows.version for windows in filter_completion(windows_completion()).values())
