@@ -19,7 +19,7 @@
 from __future__ import (absolute_import, division, print_function)
 
 from units.compat import unittest
-from units.compat.mock import MagicMock
+from unittest.mock import MagicMock
 
 from ansible.executor.task_queue_manager import TaskQueueManager
 from ansible.playbook import Playbook
